/**
 * Copyright 2005 Darren L. Spurgeon
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ajaxtags.tags;

import java.io.IOException;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.TagSupport;

import org.apache.taglibs.standard.lang.support.ExpressionEvaluatorManager;

/**
 * Tag handler for AJAX tabbed panel.
 *
 * @author Darren Spurgeon
 * @version $Revision$ $Date$
 */
public class AjaxTabPanelTag extends TagSupport {

  private String panelStyleId;

  private String contentStyleId;

  private String defaultTabBaseUrl;

  private String currentStyleId;

  private String defaultTabParameters = "";

  private String postFunction;

  private String emptyFunction;

  private String errorFunction;

  public String getContentStyleId() {
    return contentStyleId;
  }

  public void setContentStyleId(String contentStyleId) {
    this.contentStyleId = contentStyleId;
  }

  public String getCurrentStyleId() {
    return currentStyleId;
  }

  public void setCurrentStyleId(String currentStyleId) {
    this.currentStyleId = currentStyleId;
  }

  public String getDefaultTabParameters() {
    return defaultTabParameters;
  }

  public void setDefaultTabParameters(String defaultTabParameters) {
    this.defaultTabParameters = defaultTabParameters;
  }

  public String getDefaultTabBaseUrl() {
    return defaultTabBaseUrl;
  }

  public void setDefaultTabBaseUrl(String defaultTab) {
    this.defaultTabBaseUrl = defaultTab;
  }

  public String getEmptyFunction() {
    return emptyFunction;
  }

  public void setEmptyFunction(String emptyFunction) {
    this.emptyFunction = emptyFunction;
  }

  public String getErrorFunction() {
    return errorFunction;
  }

  public void setErrorFunction(String errorFunction) {
    this.errorFunction = errorFunction;
  }

  public String getPanelStyleId() {
    return panelStyleId;
  }

  public void setPanelStyleId(String panelStyleId) {
    this.panelStyleId = panelStyleId;
  }

  public String getPostFunction() {
    return postFunction;
  }

  public void setPostFunction(String postFunction) {
    this.postFunction = postFunction;
  }

  public int doStartTag() throws JspException {
    // Required Properties
    this.panelStyleId = (String) ExpressionEvaluatorManager.evaluate("panelStyleId",
        this.panelStyleId, String.class, this, super.pageContext);
    this.contentStyleId = (String) ExpressionEvaluatorManager.evaluate("contentStyleId",
        this.contentStyleId, String.class, this, super.pageContext);
    this.currentStyleId = (String) ExpressionEvaluatorManager.evaluate("currentStyleId",
        this.currentStyleId, String.class, this, super.pageContext);

    // Optional Properties
    if (this.postFunction != null) {
      this.postFunction = (String) ExpressionEvaluatorManager.evaluate("postFunction",
          this.postFunction, String.class, this, super.pageContext);
    }
    if (this.errorFunction != null) {
      this.errorFunction = (String) ExpressionEvaluatorManager.evaluate("errorFunction",
          this.errorFunction, String.class, this, super.pageContext);
    }
    if (this.emptyFunction != null) {
      this.emptyFunction = (String) ExpressionEvaluatorManager.evaluate("emptyFunction",
          this.emptyFunction, String.class, this, super.pageContext);
    }

    StringBuffer script = new StringBuffer();
    script.append("<div id=\"");
    script.append(this.panelStyleId);
    script.append("\">\n<ul>");
    JspWriter writer = pageContext.getOut();
    try {
      writer.println(script);
    } catch (IOException e) {
      throw new JspException(e.getMessage());
    }
    return EVAL_BODY_INCLUDE;
  }

  public int doEndTag() throws JspException {
    OptionsBuilder options = new OptionsBuilder();
    options.add("source", "elem", false);
    options.add("target", this.contentStyleId, true);
    options.add("currentStyleId", this.currentStyleId, true);
    options.add("parameters", "params", false);
    if (this.postFunction != null)
      options.add("postFunction", this.postFunction, false);
    if (this.emptyFunction != null)
      options.add("emptyFunction", this.emptyFunction, false);
    if (this.errorFunction != null)
      options.add("errorFunction", this.errorFunction, false);

    StringBuffer script = new StringBuffer();
    script.append("</ul>\n</div>\n<div id=\"");
    script.append(this.contentStyleId);
    script.append("\"></div>\n");
    script.append("<script type=\"text/javascript\">\n");
    script.append("function executeAjaxTab(elem, url, params) {\n");
    script.append("  var myAjax = new AjaxJspTag.TabPanel(url, {\n");
    script.append(options.toString());
    script.append("});\n");
    script.append("}\n\n");
    script.append("addOnLoadEvent(executeAjaxTab($('");
    script.append(this.currentStyleId);
    script.append("'), '");
    script.append(this.getDefaultTabBaseUrl());
    script.append("', '");
    script.append(this.defaultTabParameters);
    script.append("'));\n");
    script.append("</script>\n\n");
    JspWriter writer = pageContext.getOut();
    try {
      writer.println(script);
    } catch (IOException e) {
      throw new JspException(e.getMessage());
    }
    return EVAL_PAGE;
  }

  public void release() {
    this.panelStyleId = null;
    this.contentStyleId = null;
    this.defaultTabBaseUrl = null;
    this.defaultTabParameters = null;
    this.currentStyleId = null;
    this.postFunction = null;
    this.emptyFunction = null;
    this.errorFunction = null;
    super.release();
  }

}
